//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://apps.jcns.fz-juelich.de/frida                                   *//
//**************************************************************************//

//! \file  axis.h
//! \brief CAxis: one axis of a coordinate frame

//! One axis of a coordinate system, for use in plotting.

class CAxis {
 public:
    double inf, sup; // defining plot range [inf..sup]
    bool   logflag;  // use logarithmic axis ?
    bool   force;    // force points into range [inf..sup] ?

    CAxis( bool _log ) :
        logflag(_log), force(false) { setAuto(); }

    void setLog( bool _log );

    void setAuto();
    void setLimits( string axnam, double _inf, double _sup );
    void setRoundedLimits( string axnam, double _inf, double _sup );
    void ask_and_set( const string& quest );

    string str() const;
    string info() const;
    bool finite() const;
    bool contains( double val ) const;
    bool close_enough( double v1, double v2, double tol ) const;
    double value2plotcoord( double v ) const;
    double value2ploterror( double v, double dv ) const;
    void set_xgrid( vector<double>& x, int n ) const;
    double pc( double v ) const;
    double pcerr( double v, double dv ) const;
    double inf_pos() const;
    void calc_ticks( vector<double>& Tacks, int *ntpt, double *ticklim )
        const;
};

