//***************************************************************************//
//* FRIDA: fast reliable interactive data analysis                          *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                 *//
//* http://apps.jcns.fz-juelich.de/frida                                    *//
//***************************************************************************//

//! \file  coord.h
//! \brief coordinates CCoord, numeric parameters CParam

//! A coordinate, consisting of a name and a unit.

class CCoord {
 public:
    string name, unit;

    CCoord() {};
    CCoord( const string _name, const string _unit ) :
        name(_name), unit(_unit) {}
    CCoord( const string in );

    void ask_and_set( const string& quest );

    friend ostream& operator<< (ostream &s, CCoord C);
    string str_std() const;
    string str_compact( int maxlen=0 ) const;
    string str_ps() const;
    string to_a01() const;
    bool defined() { return name!=""; }
    string load_a01(string);

    bool operator== (const CCoord& Co2) const { 
        return (name==Co2.name) && (unit==Co2.unit); }
    bool operator!= (const CCoord& Co2) const {
        return !(*this==Co2); }
};

//! A file-wide parameter, consisting of a coordinate and a value.

class CParam {
 public:
    CCoord Co;
    double val;

    CParam() {}
    CParam( CCoord _Co, double _val=0 ) : Co(_Co), val(_val) {}
    CParam( string _name, string _unit="", double _val=0 ) : 
        Co( CCoord(_name, _unit) ), val(_val) {}
};


