//***************************************************************************//
//* FRIDA: fast reliable interactive data analysis                          *//
//* (C) Joachim Wuttke 2001-                                                *//
//* http://apps.jcns.fz-juelich.de/frida                                    *//
//***************************************************************************//

//! \file  func.h
//! \brief collection NFunctions of function wrappers CFunc

//  Low-level function types.

typedef double (*func_f1) (double);
typedef double (*func_f2) (double, double);
typedef double (*func_f3) (double, double, double);

//  Derivatives thereof.

typedef void (*deri_f1) (double&,double&,double,double);
typedef void (*deri_f2) (double&,double&,double,double,double,double);
typedef void (*deri_f3) (double&,double&,
                         double,double,double,double,double,double);

//! A wrapper holding a function, its gradient, its name, and more.

class CFunc {
 public:
    string txt;
    int narg;
    func_f1 f1;
    func_f2 f2;
    func_f3 f3;
    deri_f1 d1;
    deri_f2 d2;
    deri_f3 d3;
    int prec;
    string com;
    class CCoord coord( class CCoord *co ) const;
    class CCoord coord( class CCoord *co1, class CCoord *co2 ) const;
    class CCoord coord( class CCoord *co1, class CCoord *co2,
                        class CCoord *co3 ) const;
    CFunc( string _txt, func_f1 _f1, deri_f1 _d1, int _prec, string _com="" )
        : txt(_txt), narg(1), f1(_f1), d1(_d1), prec(_prec), com(_com) {};
    CFunc( string _txt, func_f2 _f2, deri_f2 _d2, int _prec, string _com="" )
        : txt(_txt), narg(2), f2(_f2), d2(_d2), prec(_prec), com(_com) {};
    CFunc( string _txt, func_f3 _f3, deri_f3 _d3, int _prec, string _com="" )
        : txt(_txt), narg(3), f3(_f3), d3(_d3), prec(_prec), com(_com) {};

    void register_me() const;
};

//! Collection of function wrappers CFunc

namespace NFunctions { // public (short) interface
    void initialize();
    CFunc* find( string nam );
    string list();
}

