//***************************************************************************//
//* FRIDA: fast reliable interactive data analysis                          *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                 *//
//* http://apps.jcns.fz-juelich.de/frida                                    *//
//***************************************************************************//

//! \file  mem.h
//! \brief NOlm: online memory and default file selection

//! Online memory and the default file selection.

namespace NOlm {

    // Access online memory:
    POlo mem_get  ( int k );
    POld mem_get_D( int k );
    POlc mem_get_C( int k );

    // Modify online memory (file selection is changed accordingly):
    void mem_store( POlo olo, int k=-1 );
    void mem_store( POld old, int k=-1 );
    void mem_store( POlc olc, int k=-1 );
    void mem_del();
    void mem_del( int k );
    void mem_xchange();
    void mem_copy();

    // Access file selection:
    int sel_size();
    string sel_str();
    POlo sel_first();
    int nJ_max();

    // Modify file selection:
    void sel_push( class CList* _FSel );
    bool sel_try( const string& inp );
    void sel_collect_begin();
    void sel_collect_end();
    void sel_increment( int step );

    // Backup:
    void mem_backup_store();
    void mem_backup_retrieve();

    //! Iterate over all files in selection.
    class IterateO {
    protected:
        unique_ptr<class MyIterate> pimpl;
    public:
        IterateO( int mode=0 );
        virtual ~IterateO();
        POlo operator()();
        void reset();
        int k() const;
        int iteration() const;
        int size() const;
    };

    //! Bit values of 'mode' parameter in IterateO constructor. 
    const int IterALL = 1; ///< select all online files
    const int IterTOL = 2; ///< tolerate empty selection

    //! Iterate over data files in selection.
    class IterateD : public IterateO { 
    public:
        POld operator()();
        int size() const;
    };

    //! Iterate over curve files in selection.
    class IterateC : public IterateO {
    public:
        POlc operator()();
        int size() const;
    };

    //! Allow command to overwrite its input files?
    extern bool overwrite;
}   // namespace NOlm

