//***************************************************************************//
//* FRIDA: fast reliable interactive data analysis                          *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                 *//
//* http://apps.jcns.fz-juelich.de/frida                                    *//
//***************************************************************************//

//! \file  var.cpp
//! \brief variable reference CVar

#include <string>
#include <vector>

#include "defs.h"
#include "var.h"

//***************************************************************************//
//* class CVar                                                         *//
//***************************************************************************//

// Variables may be unnumbered (x,y,i,j,k,f,...) or numbered (z0,p0,...).

//! Auxilary routine for decoding the number withing the parameter label.

int CVar::requestNum( string s, int pos )
{
    if ( s.size()<=pos )
        throw "missing parameter number in '" + s + "'";
    int ret;
    if ( !triv::any2int( s.substr(pos), &ret ) )
        throw "invalid parameter number in '" + s + "'";
    return ret;
}


//! Decode string containing parameter label (like "x" or "p3").

CVar::CVar( string s )
{
    num = -1;
    if( s.size()<1 )
        throw "Variable name missing";
    
    if ( s.substr(0,2)=="cq" ) {     // curve quality
        num = requestNum( s, 2 );
        typ = _CQ;
    } else if ( s[0]=='p' ) {        // curve parameter
        num = requestNum( s, 1 );
        typ = _CP;
    } else if ( s=="z+" ) {          // new z value
        typ = _Z; 
    } else if ( s[0]=='z' ) {        // existing z value
        num = requestNum( s, 1 );
        typ = _Z;
    } else if ( s=="r+" ) {          // new real par
        typ = _R;
    } else if ( s[0]=='r' ) { // existing real par
        num = requestNum( s, 1 );
        typ = _R;
    } else if ( s=="nj" ) {
        typ = _NJ;
    } else if ( s=="ni" ) {
        typ = _NI;
    } else if ( s=="x" ) {
        typ = _X;
    } else if ( s=="y" ) {
        typ = _Y;
    } else if ( s=="dy" ) {
        typ = _DY;
    } else if ( s=="c" ) {
        typ = _C;
    } else if ( s=="k" ) {
        typ = _K;
    } else if ( s=="j" ) {
        typ = _J;
    } else if ( s=="i" ) {
        typ = _I;
    } else {
        throw "Invalid variable '" + s + "'";
        // typ = _NOREF;
    }
}


//! Return associated error variable.

CVar CVar::errvar() const
{
    switch (typ) {
    case _Y:
        return CVar( _DY );  break;
    default:
        return CVar();
    }
}


//! Convert to text.

string CVar::var_info() const
{
    string ret;
    switch (typ) {
    case _X:  ret = "x";  break;
    case _Y:  ret = "y";  break;
    case _DY: ret = "dy";  break;
    case _C:  ret = "c";  break;
    case _Z:  ret = "z";  break;
    case _R:  ret = "r";  break;
    case _CP: ret = "p";  break;
    case _CQ: ret = "cq"; break;
    case _K:  ret = "k";  break;
    case _J:  ret = "j";  break;
    case _I:  ret = "i";  break;
    case _NJ: ret = "nj"; break;
    case _NI: ret = "ni"; break;
    default:
        throw "BUG: no info available for unexpected variable typ";
    }
    if ( numbered() ) {
        if( num==-1 )
            ret += "+";
        else
            ret += S(num);
    }
    return ret;
}

