//***************************************************************************//
//* FRIDA: fast reliable interactive data analysis                          *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                 *//
//* http://apps.jcns.fz-juelich.de/frida                                    *//
//***************************************************************************//

//! \file  zentry.h
//! \brief z-dependent entries CZentry < CCurve, CSpec

//! Virtual base class for CCurve and CSpec, holds a z vector and more.

class CZentry {
 public:
    vector<double> z, dz;
    bool frozen;

    CZentry();
    virtual ~CZentry() {}; // magic: makes dynamic_cast possible

    void copy_z_base( boost::shared_ptr<class CZentry> ein );
    virtual string infoLine() const = 0;
};


//! A CZentry within a curve file, holds parameters and fit outcome.

class CCurve: public CZentry {
 public:
    static const int mQuality = 3;
    vector<double> P;
    vector<bool>   fixed;
    double Quality[mQuality];
    
    void clear( void );
    string infoLine() const;
};


//! A CZentry within a data file, holds a x-y-dy data set.

class CSpec: public CZentry {
 public:
    vector<double> x, y, dy;
    
    // modify:
    void push_xy( double _x, double _y );
    void push_xyd( double _x, double _y, double _dy );
    void clear(); 
    void remove( int begin, int end );
    void resize( int n, bool with_dy );
    void purge_dy();

    // extract data:
    int size() const;
    void intpol( const vector<double>& in_x, vector<double>& out_y ) const;
    double intpol( double x ) const {
        vector<double> xx(1), yy(1); xx[0] = x; intpol(xx,yy); return yy[0]; }
    double norm( string info ) const;
    bool has_step( double step ) const;
    string infoLine() const;
};


//! at global scope, for use in sorting.

bool CompareZ( const PZentry& E1, const PZentry& E2 );

